@@ -40,9 +40,6 @@
 #ifndef PROGNAME
 	#define PROGNAME "sxcs"
 #endif
-#ifndef VERSION
-	#define VERSION "v1.0.0"
-#endif
 
 #define ARRLEN(X)        (sizeof(X) / sizeof((X)[0]))
 #define MAX(A, B)        ((A) > (B) ? (A) : (B))
@@ -58,8 +55,6 @@
 
 #define FILTER_SEQ_FROM_ARRAY(X)  { X, ARRLEN(X) }
 
-#define S(X)       str_c89_workaround((char *)(X), sizeof(X) - 1)
-
 /* portable compiler attributes */
 #ifndef __has_attribute
 	#define __has_attribute(X) (0)
@@ -211,17 +206,6 @@ fatal(const char *fmt, ...)
 	exit(1);
 }
 
-/* c89 no compound literal support */
-static Str
-str_c89_workaround(char *s, ptrdiff_t len)
-{
-	Str ret;
-	ASSERT(s != NULL && len >= 0);
-	ret.s = (uchar *)s;
-	ret.len = len;
-	return ret;
-}
-
 static Str
 str_from_cstr(char *s)
 {
@@ -383,7 +367,7 @@ static void
 version(void)
 {
 	char s[] =
-		PROGNAME" "VERSION"\n\n"
+		PROGNAME" v1.1.0\n\n"
 		"Copyright (C) 2022-2024 NRK and contributors.\n"
 		"License: GPLv3+ <https://gnu.org/licenses/gpl.html>.\n"
 		"Upstream: <https://codeberg.org/NRK/sxcs>\n"
@@ -425,42 +409,50 @@ filter_parse(Str arg)
 	filter = &fs_buf;
 }
 
+/* inspired by https://github.com/skeeto/scratch/blob/master/parsers/imgo.c */
+typedef struct { char **argv, *cur, *flag; int len; } OptCtx;
+#define OPT(O, SO, LO) ( ((O)->len == 1 && (SO) != 0x0 && (O)->flag[0] == (SO)) || \
+	((LO) != 0 && (O)->len-1 == sizeof(LO)-1 && memcmp(LO, (O)->flag+1, (O)->len-1) == 0) )
+static int
+opt_next(OptCtx *o)
+{
+	if (o->cur == NULL || *o->cur == '\0') {
+		if ((o->cur = *o->argv++) == NULL || *o->cur++ != '-' || *o->cur == '\0')
+			return --o->argv, 0;
+		if (*o->cur == '-') {
+			o->flag = o->cur; o->cur = NULL;
+			return o->len = (int)strlen(o->flag);
+		}
+	}
+	o->flag = o->cur++;
+	return o->len = 1;
+}
+
 static Options
 opt_parse(int argc, char *argv[])
 {
-	int i;
 	Options ret = {0};
-	int no_color = 0;
+	int fmt_default = 1;
+	OptCtx o[1] = {0};
 
-	for (i = 1; i < argc; ++i) {
-		Str a = str_from_cstr(argv[i]);
-		if (str_eq(a, S("--rgb")))
-			ret.fmt |= OUTPUT_RGB;
-		else if (str_eq(a, S("--hex")))
-			ret.fmt |= OUTPUT_HEX;
-		else if (str_eq(a, S("--hsl")))
-			ret.fmt |= OUTPUT_HSL;
-		else if (str_eq(a, S("--color-none")))
-			no_color = 1;
-		else if (str_eq(a, S("--one-shot")) || str_eq(a, S("-o")))
-			ret.oneshot = 1;
-		else if (str_eq(a, S("--quit-on-keypress")) || str_eq(a, S("-q")))
-			ret.quit_on_keypress = 1;
-		else if (str_eq(a, S("--keyboard")) || str_eq(a, S("-k")))
-			ret.keyboard = 1;
-		else if (str_eq(a, S("--mag-none")))
-			ret.no_mag = 1;
-		else if (str_eq(a, S("--mag-filters")))
-			filter_parse(str_from_cstr(argv[++i]));
-		else if (str_eq(a, S("--help")) || str_eq(a, S("-h")))
-			usage();
-		else if (str_eq(a, S("--version")))
-			version();
-		else
-			fatal("unknown argument `%s`.", argv[i]);
-	}
+	for (o->argv = argv + (argc > 0); opt_next(o);) { /* NOLINTBEGIN(*misleading-indentation) */
+		     if (OPT(o, 0x0, "rgb"))  ret.fmt |= OUTPUT_RGB;
+		else if (OPT(o, 0x0, "hex"))  ret.fmt |= OUTPUT_HEX;
+		else if (OPT(o, 0x0, "hsl"))  ret.fmt |= OUTPUT_HSL;
+		else if (OPT(o, 0x0, "color-none"))  ret.fmt = fmt_default = 0;
+		else if (OPT(o, 'o', "one-shot"))    ret.oneshot = 1;
+		else if (OPT(o, 'q', "quit-on-keypress"))  ret.quit_on_keypress = 1;
+		else if (OPT(o, 'k', "keyboard"))  ret.keyboard = 1;
+		else if (OPT(o, 0x0, "mag-none"))  ret.no_mag = 1;
+		else if (OPT(o, 0x0, "mag-filters"))  filter_parse(str_from_cstr(*o->argv++));
+		else if (OPT(o, 'h', "help"))     usage();
+		else if (OPT(o, 0x0, "version"))  version();
+		else fatal("unknown argument `-%.*s`", (int)o->len, o->flag);
+	} /* NOLINTEND(*misleading-indentation) */
+	if (*o->argv)
+		fatal("excess argument: `%s`", *o->argv);
 
-	if (ret.fmt == OUTPUT_NONE && !no_color)
+	if (ret.fmt == OUTPUT_NONE && fmt_default)
 		ret.fmt = OUTPUT_DEFAULT;
 
 	if (ret.quit_on_keypress && ret.keyboard)
